#coding: utf-8

from django.test import Client
from django.conf import settings
from django.contrib.auth import login
from django.http import HttpRequest
from django.contrib.auth.models import User
from django.utils.importlib import import_module


class DjangoClientSender(object):
    u'''
    Механизм непосредственной отправки запросов к удаленному серверу.

    .. method:: DjangoClientSender.login_user(self, username)

    Метод имитации входа пользователя в систему, без использования пароля
    и с созданием сессии.

    :param username: Имя существующего пользователя

    .. method:: DjangoClientSender.send_request(self, req)

    Метод отсылки сохраненного запроса и проверки возвращенного результата.

    :param req: Объект сохраненного запроса.

    '''

    def __init__(self):
        self.client = Client(HTTP_USER_AGENT='Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36'\
                                             ' (KHTML, like Gecko) Chrome/29.0.1547.66 Safari/537.36')
        self.session_id = None

    def login_user(self, username):
        user = User.objects.get(username=username)
        user.backend = 'django.contrib.auth.backends.ModelBackend'
        session_engine = import_module(settings.SESSION_ENGINE)

        # Создаем сессию для хранения пользовательских данных
        req = HttpRequest()
        req.session = session_engine.SessionStore()
        # Регистрируем вход в систему
        login(req, user)
        req.session.save()

        return req.session.session_key

    def send_request(self, req):
        method_map = {
            'GET': self.client.get,
            'POST': self.client.post,
            'HEAD': self.client.head
        }

        # Так как нам недоступны middleware во время отсылки запроса,
        # мы должны сами создать сессию и зарегистрировать вход пользователя
        # в систему.
        if not self.session_id:
            self.session_id = self.login_user(req.auth['username'])
            session_cookie = settings.SESSION_COOKIE_NAME
            self.client.cookies[session_cookie] = self.session_id
            cookie_data = {
                'max-age': None,
                'path': '/',
                'domain': settings.SESSION_COOKIE_DOMAIN,
                'secure': settings.SESSION_COOKIE_SECURE or None,
                'expires': None,
            }
            self.client.cookies[session_cookie].update(cookie_data)

        handler = method_map[req.verb]
        resp = handler(req.uri,
            data=req.body,
            content_type=req.headers.get('Content-Type', 'application/x-www-form-urlencoded; charset=UTF-8'))
        return resp.status_code, resp.content
